6.52   os16: getopt(3)

NOME

    getopt - scansione delle opzioni della riga di comando

SINTASSI

    #include <unistd.h>
    extern *char optarg;
    extern int   optind;
    extern int   opterr;
    extern int   optopt;
    int getopt (int <argc>, char *const <argv[]>, const char *<optstring>);

DESCRIZIONE

    La funzione <getopt()> riceve, come primi due argomenti, gli stessi
    parametri <argc> e <argv[]>, che sono gia` della funzione <main()> del
    programma in cui <getopt()> si usa. In altri termini, <getopt()> deve
    conoscere la quantita` degli argomenti usati per l'avvio del programma e
    deve poterli scandire. L'ultimo argomento di <getopt()> e` una stringa
    contenente l'elenco delle lettere delle opzioni che ci si attende di
    trovare nella scansione delle stringhe dell'array <argv[]>, con altre sigle
    eventuali per sapere se tali opzioni sono singole o si attendono un proprio
    argomento.

    Per poter usare la funzione <getopt()> proficuamente, e` necessario che la
    sintassi di utilizzo del programma del quale si vuole scandire la riga di
    comando, sia uniforme con l'uso comune:

    <programma> [-<x>[ <argomento>]]... [<argomento>]...

    Pertanto, dopo il nome del programma possono esserci delle opzioni,
    riconoscibili perche' composte da una sola lettera, preceduta da un
    trattino. Tali opzioni potrebbero richiedere un proprio argomento. Dopo le
    opzioni e i relativi argomenti, ci possono essere altri argomenti, al di
    fuori della competenza di <getopt()>. Vale anche la considerazione che piu`
    opzioni, prive di argomento, possono essere unite assieme in un'unica
    parola, con un solo trattino iniziale.

    La funzione <getopt()> si avvale di variabili pubbliche, di cui occorre
    conoscere lo scopo.

    La variabile <optind> viene usata da <getopt()> come indice per scandire l'
    array <argv[]>. Quando con gli utilizzi successivi di <optarg()> si
    determina che e` stata completata la scansione delle opzioni (in quanto
    <optarg()> restituisce il valore -1), la variabile <optind> diventa utile
    per conoscere qual e` il prossimo elemento di <argv[]> da prendere in
    considerazione, trattandosi del primo argomento della riga di comando che
    non e` un'opzione.

    La variabile <opterr> serve per configurare il comportamento di <getopt()>.
    Questa variabile contiene inizialmente il valore 1. Quando <getopt()>
    incontra un'opzione per la quale si richiede un argomento, il quale risulta
    pero` mancante, se la variabile <opterr> risulta avere un valore diverso da
    zero, visualizza un messaggio di errore attraverso lo standard error.
    Pertanto, per evitare tale visualizzazione, e` sufficiente assegnare
    preventivamente il valore zero alla variabile <opterr>.

    Quando un'opzione individuata da <getopt()> risulta errata per qualche
    ragione (perche' non prevista o perche' si attende un argomento che invece
    non c'e`), la variabile <optopt> riceve il valore (tradotto da carattere
    senza segno a intero) della lettera corrispondente a quell'opzione.
    Pertanto, in tal modo e` possibile conoscere cosa ha provocato il problema.

    Il puntatore <optarg> viene modificato quando <getopt()> incontra
    un'opzione che chiede un argomento. In tal caso, <optarg> viene modificato
    in modo da puntare alla stringa che rappresenta tale argomento.

    La compilazione della stringa corrispondente a <optstring> deve avvenire
    secondo una sintassi precisa:

    [:][<x>[:]]...

    La stringa <optstring> puo` iniziare con un simbolo di due punti, quindi
    seguono le lettere che rappresentano le opzioni possibili, tenendo conto
    che quelle per cui si attende un argomento devono anche essere seguite da
    due punti. Per esempio, ab:cd: significa che ci puo` essere un'opzione -a,
    un'opzione -b seguita da un argomento, un'opzione -c e un'opzione -d
    seguita da un argomento.

    Per comprendere l'uso della funzione <getopt()> si propone una versione
    ultraridotta di kill(1) [4.10], dove si ammette solo l'invio dei segnali
    SIGTERM e SIGQUIT.

    #include <sys/os16.h>
    #include <sys/stat.h>
    #include <sys/types.h>
    #include <unistd.h>
    #include <stdlib.h>
    #include <fcntl.h>
    #include <errno.h>
    #include <signal.h>
    #include <stdio.h>
    #include <string.h>
    #include <limits.h>
    #include <libgen.h>
    //----------------------------------------------------------------------
    int
    main (int argc, char *argv[], char *envp[])
    {
        int          signal = SIGTERM;
        int          pid;
        int          a;             // Index inside arguments.
        int          opt;
        extern char *optarg;
        extern int   optopt;
        //
        while ((opt = getopt (argc, argv, ":ls:")) != -1)
          {
            switch (opt)
              {
                case 'l':
                    printf ("TERM ");
                    printf ("KILL ");
                    printf ("\n");
                    return (0);
                    break;
                case 's':
                    if (strcmp (optarg, "KILL") == 0)
                      {
                        signal = SIGKILL;
                      }
                    else if (strcmp (optarg, "TERM") == 0)
                      {
                        signal = SIGTERM;
                      }
                    break;
                case '?':
                    fprintf (stderr, "Unknown option -%c.\n", optopt);
                    return (1);
                    break;
                case ':':
                    fprintf (stderr, "Missing argument for option "
                                     "-%c\n", optopt);
                    return (1);
                    break;
                default:
                    fprintf (stderr, "Getopt problem: unknown option "
                                     "%c\n", opt);
                    return (1);
              }
          }
        //
        // Scan other command line arguments.
        //
        for (a = optind; a < argc; a++)
          {
            pid = atoi (argv[a]);
            if (pid > 0)
              {
                if (kill (pid, signal) < 0)
                  {
                    perror (argv[a]);
                  }
              }
          }
        return (0);
    }

    Come si vede nell'esempio, la funzione <getopt()> viene chiamata sempre
    nello stesso modo, all'interno di un ciclo iterativo.

    Alla prima chiamata della funzione, questa esamina il primo argomento della
    riga di comando, verificando se si tratta di un'opzione o meno. Se si
    tratta di un'opzione, benche' possa essere errata per qualche ragione,
    restituisce un carattere (convertito a intero), il quale puo` corrispondere
    alla lettera dell'opzione se questa e` valida, oppure a un simbolo
    differente in caso di problemi. Nelle chiamate successive, <getopt()>
    considera di volta in volta gli argomenti successivi della riga di comando,
    fino a quando si accorge che non ci sono piu` opzioni e restituisce
    semplicemente il valore -1.

    Durante la scansione delle opzioni, se <getopt()> restituisce il carattere
    ?, significa che ha incontrato un'opzione errata: potrebbe trattarsi di
    un'opzione non prevista, oppure di un'opzione che attende un argomento che
    non c'e`. Tuttavia, la stringa <optstring> potrebbe iniziare opportunamente
    con il simbolo di due punti, cosi` come si vede nell'esempio. In tal caso,
    se <getopt()> incontra un'opzione errata in quanto mancante di un'opzione
    necessaria, invece di restituire ?, restituisce :, cosi` da poter
    distinguere il tipo di errore.

    E` il caso di osservare che le chiamate successive di <getopt()> fanno
    progredire la scansione della riga di comando e generalmente non c'e`
    bisogno di tornare indietro per ripeterla. Tuttavia, nel caso lo si
    volesse, basterebbe reinizializzare la variabile <optind> a uno (il primo
    argomento della riga di comando).

FILE SORGENTI

    lib/unistd.h [13.17]

    lib/unistd/getopt.c [13.17.20]

